package ru.myitschool.work.data;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import ru.myitschool.work.data.dto.HistoryDto;
import ru.myitschool.work.data.dto.UserDto;
import ru.myitschool.work.data.network.RetrofitFactory;
import ru.myitschool.work.data.source.CredentialsDataSource;
import ru.myitschool.work.data.source.UserApi;
import ru.myitschool.work.domain.UserRepository;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.Status;
import ru.myitschool.work.domain.entities.UserEntity;
import ru.myitschool.work.domain.sign.SignUserRepository;

public class UserRepositoryImpl implements UserRepository, SignUserRepository {

    private static UserRepositoryImpl INSTANCE;
    private UserApi userApi = RetrofitFactory.getInstance().getUserApi();
    private final CredentialsDataSource credentialsDataSource = CredentialsDataSource.getInstance();

    private UserRepositoryImpl() {}

    public static synchronized UserRepositoryImpl getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new UserRepositoryImpl();
        }
        return INSTANCE;
    }


    /*@Override
    public void getAllUsers(@NonNull Consumer<Status<List<UserEntity>>> callback) {
        userApi.getAllUsers().enqueue(new CallToConsumer<>(
                callback,
                usersDto -> {
                    ArrayList<UserEntity> result = new ArrayList<>(usersDto.size());
                    for (UserDto user : usersDto) {
                        final String username = user.username;
                        final String photo = user.photo;
                        final Long lastVisit = user.lastVisit;
                        final Integer idUser = user.idUser;
                        final String position = user.position;
                        if (idUser != null && username != null) {
                            result.add(new UserEntity(username, photo, lastVisit, idUser, position));
                        }
                    }
                    return result;
                }
        ));

    }*/

    @Override
    public void getAllUserHistory(@NonNull String username, @NonNull Consumer<Status<List<HistoryEntity>>> callback) {
        userApi.getAllUserHistory(username).enqueue(new CallToConsumer<>(
                callback,
                historiesDto -> {
                    ArrayList<HistoryEntity> result1 = new ArrayList<>(historiesDto.size());
                    for (HistoryDto history : historiesDto) {
                        final Long id1 = history.id;
                        final String username1 = history.username;
                        final String time = history.time;
                        final String type = history.type;
                        final Long code = history.code;
                        if (username1 != null && id1 != null){
                            result1.add(new HistoryEntity(id1, username1, type, time, code));
                        }
                    }
                    return result1;
                }
        ));

    }

    @Override
    public void isExistUser(@NonNull String username, Consumer<Status<Void>> callback) {
        userApi.isExistUser(username).enqueue(new CallToConsumer<>(
                callback,
                dto -> null
        ));
    }

    /* public void getHistoryById(@NonNull Integer id, @NonNull Consumer<Status<List<HistoryEntity>>> callback){
          userApi.getHistoryById(id).enqueue(new CallToConsumer<>(
                  callback,
                  historiesDto -> {
                      final Integer id1 = historiesDto.id;
                      final Integer idUser = historiesDto.idUser;
                      final Long time = historiesDto.time;
                      final String nameReader = historiesDto.nameReader;
                      final String type = historiesDto.type;
                      if (id1 != null && idUser != null){
                          HistoryEntity historyEntity = new HistoryEntity(id1, idUser, time, nameReader, type);
                          return historyEntity;
                      }
                      else{
                          return null;
                      }

      };
      } */

    @Override
    public void getUserByUsername(@NonNull String username, @NonNull Consumer<Status<UserEntity>> callback) {
        userApi.getUserByUsername(username).enqueue(new CallToConsumer<>(
                callback,
                user -> {
                    if (user != null) {
                        final Integer id1 = user.id;
                        final String username1 = user.username;
                        final String name = user.name;
                        final String photo = user.photo;
                        final String position = user.position;
                        final String authority = user.authority;
                        if (id1 != null && username != null){
                            return new UserEntity(id1, username1, name, photo, position, authority);
                        }
                        else{
                            return null;
                        }
                    }
                    else{
                        return null;
                    }
                }
        ));

    }

    public void login(@NonNull String username, @NonNull String password, Consumer<Status<Void>> callback) {
        credentialsDataSource.updateLogin(username, password);
        userApi = RetrofitFactory.getInstance().getUserApi();
        userApi.login().enqueue(new CallToConsumer<>(
                callback,
                dto -> null
        ));
    }



    public void logout() {
        credentialsDataSource.logout();
    }

}
