package ru.myitschool.work.ui.admin;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import java.util.List;

import ru.myitschool.work.data.UserRepositoryImpl;
import ru.myitschool.work.domain.GetAllUserHistoryUseCase;
import ru.myitschool.work.domain.GetUserByUsernameUseCase;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.Status;
import ru.myitschool.work.domain.sign.IsUserExistUseCase;
import ru.myitschool.work.ui.information.InformationViewModel;
import ru.myitschool.work.ui.profile.ProfileViewModel;

public class AdminViewModel extends ViewModel {
    private final MutableLiveData<AdminViewModel.State> mutableStateLiveData = new MutableLiveData<>();

    public final LiveData<AdminViewModel.State> stateLiveData = mutableStateLiveData;

    public final GetUserByUsernameUseCase getUserByUsernameUseCase = new GetUserByUsernameUseCase(
            UserRepositoryImpl.getInstance()
    );

    public final GetAllUserHistoryUseCase getAllUserHistoryListUseCase = new GetAllUserHistoryUseCase(
            UserRepositoryImpl.getInstance()
    );
    private final IsUserExistUseCase isUserExistUseCase = new IsUserExistUseCase(
            UserRepositoryImpl.getInstance()
    );

    public void load(@NonNull String username) {
        mutableStateLiveData.setValue(new AdminViewModel.State(null, null,  true));
        postUserByUsername(username);
    }

    private void postUserByUsername(String username) {
        final String currentUsername = username;
        if (currentUsername == null || currentUsername.isEmpty()) {
            return;
        }
        isUserExistUseCase.execute(currentUsername, status -> {
            if (status.getValue() == null || status.getErrors() != null) {
                Log.d("tagg", status.getErrors().toString());
            }

        });
        getUserByUsernameUseCase.execute(currentUsername, status ->{
            if (status.getValue() == null || status.getErrors() != null) {
                Log.d("tagg", status.getErrors().toString());
            }
        });
    }


    public class State {
        @Nullable
        private final String errorMessage;

        @Nullable
        private final List<HistoryEntity> itemsHistory;

        private final boolean isLoading;

        public State(@Nullable String errorMessage, @Nullable List<HistoryEntity> itemsHistory, boolean isLoading) {
            this.errorMessage = errorMessage;
            this.itemsHistory = itemsHistory;
            this.isLoading = isLoading;
        }

        @Nullable
        public String getErrorMessage() {
            return errorMessage;
        }

        @Nullable
        public List<HistoryEntity> getItemsHistory() {
            return itemsHistory;
        }

        public boolean isLoading() {
            return isLoading;
        }
    }
}
