package ru.myitschool.work.ui.information;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.Navigation;

import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import ru.myitschool.work.R;
import ru.myitschool.work.databinding.FragmentInformationBinding;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.UserEntity;
import ru.myitschool.work.utils.Constants;
import ru.myitschool.work.ui.History;
import ru.myitschool.work.ui.StoreAPI;
import ru.myitschool.work.ui.User;
import ru.myitschool.work.ui.qr.scan.QrScanDestination;
import ru.myitschool.work.utils.PreferenceManager;
import ru.myitschool.work.utils.Utils;

public class InformationFragment extends Fragment {
    FragmentInformationBinding binding;
    PreferenceManager preferenceManager;
    InformationViewModel viewModel;
    List<HistoryEntity> history = new ArrayList<>();
    private boolean loadingHistory;
    private String username;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentInformationBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        //sharedPreferences = getContext().getSharedPreferences(getString(R.string.app_name), Context.MODE_PRIVATE);
        preferenceManager = new PreferenceManager(this.getContext());
        username = preferenceManager.getString(Constants.KEY_USER_USERNAME);
        Toast.makeText(getContext(), username, Toast.LENGTH_SHORT).show();
        viewModel = new ViewModelProvider(this).get(InformationViewModel.class);
        //getInformation();

//        binding.scan.setOnClickListener(view1 -> {
//            onClickScan(view);
//        });

        binding.logout.setOnClickListener(view2 -> {
            onClickLogout(view);
        });

        binding.refreshLayout.setOnRefreshListener(() -> {
            viewModel.load();
        });
        //TODO: SERVER ZAGRUZKA LISTAAAAAAAAAAAAAAA
        HistoryAdapter adapter = new HistoryAdapter();
        binding.historylist.setAdapter(adapter);

        subscribe(viewModel, adapter, view);

        //getParentFragmentManager()
        requireActivity().getSupportFragmentManager().setFragmentResultListener(QrScanDestination.REQUEST_KEY, getViewLifecycleOwner(), (requestKey, result) -> {
            String res = "";
            if (result.get(requestKey) != null) {
                res = result.get(requestKey).toString();
            }
            //Toast.makeText(getContext(), res, Toast.LENGTH_LONG).show();



            //ResultFragment resFragment = ResultFragment.newInstance(res);
            Navigation.findNavController(view).navigate(R.id.action_informationFragment_to_resultFragment);
            //getFragmentManager().beginTransaction().replace(R.id.nav_host_fragment, resFragment).commit();
        });

        viewModel.changeSelectedUsername(username);
        viewModel.load();
    }

    /*private void onClickRefresh() {
        getInformation();
    }*/

    private void onClickScan(View view) {
        Navigation.findNavController(view).navigate(R.id.action_informationFragment_to_qrScanFragment);
        //getFragmentManager().beginTransaction().replace(R.id.nav_host_fragment, new QrScanFragment()).commit();
    }

    private void onClickLogout(View view) {
        preferenceManager.clear();
        Navigation.findNavController(view).navigate(R.id.action_informationFragment_to_loginFragment);
        //getFragmentManager().beginTransaction().replace(R.id.nav_host_fragment, new LoginFragment()).commit();
    }

    private void subscribe(final InformationViewModel viewModel, final HistoryAdapter historyAdapter, View view) {
        viewModel.stateAllUserHistoryLiveData.observe(getViewLifecycleOwner(), state -> {
            loadingHistory = state.isLoading();
            //Toast.makeText(this.getContext(), "Zashol", Toast.LENGTH_SHORT).show();



            //binding.loading.setVisibility(Utils.visibleOrGone(loadingHistory));
            if (state.isLoading() == false) {
                binding.refreshLayout.setRefreshing(false);
                if (state.getItemsHistory() != null) {
                    history = state.getItemsHistory();

                    //TODO: BINDING


                    //binding.address.setText(history.getAddress());
                    //binding.countVolunteers.setText("Количество волонтёров: " + history.getUsers().size());
                    //List<ItemUserEntity> itemUserEntities = new ArrayList<>();
                    //for (FullUserEntity el : volunteerCenter.getUsers()) {
                    //    UserEntity user = new ItemUserEntity(el.getId(), el.getLastName(), el.getFirstName());
                    //    itemUserEntities.add(user);
                    //}
                    historyAdapter.updateData(history);
                    historyAdapter.notifyDataSetChanged();
                    binding.refreshLayout.setRefreshing(false);
                } else {
                    Toast.makeText(this.getContext(), "Error", Toast.LENGTH_SHORT).show();
                    takeError();
                }
            }
            //binding.refreshLayout.setRefreshing(false);
        });
        /*
        viewModel.stateHistoryLiveData.observe(getViewLifecycleOwner(), state -> {
            loadingHistory = state.isLoading();
            //Toast.makeText(this.getContext(), "vse", Toast.LENGTH_SHORT).show();
            binding.loading.setVisibility(Utils.visibleOrGone(loadingHistory));
            if (state.isLoading() == false) {
                if (state.getItemsHistory() != null) {
                    volunteerCenters = state.getItemsVolunteerCenters();

                    List<String> volunteerCentersString =  new ArrayList<String>();
                    for (ItemVolunteerCenterEntity el : volunteerCenters) {
                        volunteerCentersString.add(el.getAddress());
                    }

                    ArrayAdapter<String> spinnerAdapter;
                    spinnerAdapter = new ArrayAdapter<String>(this.getContext(), android.R.layout.simple_spinner_item, volunteerCentersString);

                    spinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

                    binding.spinner.setAdapter(spinnerAdapter);
                    //binding.spinner.setPromptId(R.string.chooseday);
                    //binding.spinner.setSelection(1, true);
                    String volunteerCenterId = preferenceManager.getString(Constants.KEY_VOLUNTEER_CENTER_ID);

                    if (volunteerCenterId != null) {
                        //Toast.makeText(this.getContext(), volunteerCenterId, Toast.LENGTH_SHORT).show();


                        binding.spinner.setSelection(Integer.parseInt(volunteerCenterId) - 1, true);
                    }


                } else {
                    Toast.makeText(this.getContext(), "Error", Toast.LENGTH_SHORT).show();
                }
            }





            /*boolean isSuccess = !state.isLoading()
                    && state.getErrorMessage() == null
                    && state.getItems() != null;
            //binding.refresh.setEnabled(!state.isLoading());
            //if (!state.isLoading()) //binding.refresh.setRefreshing(false);
            binding.recycler.setVisibility(Utils.visibleOrGone(isSuccess));
            binding.error.setVisibility(Utils.visibleOrGone(state.getErrorMessage() != null));
            binding.loading.setVisibility(Utils.visibleOrGone(state.isLoading()));

            binding.error.setText(state.getErrorMessage());
            if (isSuccess) {
                listAdapter.updateData(state.getItems());
            }
        });
*/
    }

    /*private void getInformation() {
        Retrofit retrofit = new Retrofit.Builder()
                .baseUrl(ru.myitschool.work.core.Constants.SERVER_ADDRESS)
                .addConverterFactory(GsonConverterFactory.create())
                .build();

        StoreAPI storeApi = retrofit.create(StoreAPI.class);

        String login = sharedPreferences.getString(Constants.KEY_LOGIN, "");

        Call<User> call = storeApi.informationUser(login);
        call.enqueue(new Callback<User>() {
            @Override
            public void onResponse(Call<User> call, Response<User> response) {
                if (response.isSuccessful()) {
                    if (response.body() != null) {
                        User user = response.body();
                        binding.fullname.setText(user.getName());
                        binding.position.setText(user.getPosition());
                        binding.lastEntry.setText(user.getLastVisit().substring(0, 10) + " " + user.getLastVisit().substring(11, 16));
                        Picasso.get().load(user.getPhoto()).into(binding.photo);
                        binding.refreshLayout.setRefreshing(false);
                    } else {
                        takeError();
                    }
                } else if (response.code() == 401) {
                    takeError();
                } else if (response.code() == 400) {
                    takeError();
                } else {
                    takeError();
                }
            }

            @Override
            public void onFailure(Call<User> call, Throwable t) {
                takeError();
            }
        });
    }*/

    private void takeError() {
        //binding.photo.setVisibility(View.GONE);
        binding.fullname.setVisibility(View.GONE);
        binding.lastEntry.setVisibility(View.GONE);
        binding.position.setVisibility(View.GONE);
        //binding.logout.setVisibility(View.GONE);
        //binding.scan.setVisibility(View.GONE);

        binding.error.setVisibility(View.VISIBLE);
        binding.refreshLayout.setRefreshing(false);
    }
}
