package ru.myitschool.work.ui.information;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import java.util.List;

import ru.myitschool.work.data.UserRepositoryImpl;
import ru.myitschool.work.domain.GetAllUserHistoryUseCase;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.Status;

public class InformationViewModel extends ViewModel {
    private final MutableLiveData<State> mutableStateHistoryLiveData = new MutableLiveData<>();

    public final LiveData<State> stateHistoryLiveData = mutableStateHistoryLiveData;

    private final MutableLiveData<State> mutableStateAllUserHistoryLiveData = new MutableLiveData<>();

    public final LiveData<State> stateAllUserHistoryLiveData = mutableStateAllUserHistoryLiveData;

    public final GetAllUserHistoryUseCase getAllUserHistoryUseCase = new GetAllUserHistoryUseCase(
            UserRepositoryImpl.getInstance()
    );




    private String username;

    public void changeSelectedUsername(@NonNull String username) {
        this.username = username;
    }

    public void load() {
        mutableStateAllUserHistoryLiveData.setValue(new InformationViewModel.State(null, null, true));
        getHistById();
        /*getAllUserHistoryUseCase.execute(username, status -> {
            mutableStateHistoryLiveData.postValue(fromStatus(status));
        });*/
    }

    public void getHistById() {
        getAllUserHistoryUseCase.execute(username, status -> {
            //Log.d("taggis", status.getStatusCode() + " " + status.getErrors());
            if (status.getStatusCode() == 200 && status.getErrors() == null) {
                mutableStateAllUserHistoryLiveData.postValue(new InformationViewModel.State(
                        status.getErrors() != null ? status.getErrors().getLocalizedMessage() : null,
                        status.getValue(),
                        false
                ));
            } else {
                //mutableErrorLiveData.postValue("Something wrong");
            }
        });
    }

    private InformationViewModel.State fromStatus(Status<List<HistoryEntity>> status) {
        return new InformationViewModel.State(
                status.getErrors() != null ? status.getErrors().getLocalizedMessage() : null,
                status.getValue(),
                false
        );
    }

    public class State {
        @Nullable
        private final String errorMessage;

        @Nullable
        private final List<HistoryEntity> itemsHistory;

        private final boolean isLoading;

        public State(@Nullable String errorMessage, @Nullable List<HistoryEntity> itemsHistory, boolean isLoading) {
            this.errorMessage = errorMessage;
            this.itemsHistory = itemsHistory;
            this.isLoading = isLoading;
        }

        @Nullable
        public String getErrorMessage() {
            return errorMessage;
        }

        @Nullable
        public List<HistoryEntity> getItemsHistory() {
            return itemsHistory;
        }

        public boolean isLoading() {
            return isLoading;
        }
    }
}
