package ru.myitschool.work.ui.login;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.Navigation;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import ru.myitschool.work.R;
import ru.myitschool.work.databinding.FragmentLoginBinding;
import ru.myitschool.work.ui.StoreAPI;
import ru.myitschool.work.ui.login.LoginViewModel;
import ru.myitschool.work.utils.Constants;
import ru.myitschool.work.utils.OnChangeText;
import ru.myitschool.work.utils.PreferenceManager;

public class LoginFragment extends Fragment {

    private FragmentLoginBinding binding;
    private PreferenceManager preferenceManager;

    private LoginViewModel viewModel;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentLoginBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        preferenceManager = new PreferenceManager(this.getContext());
        viewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        binding.login.setEnabled(false);

        binding.username.addTextChangedListener(new OnChangeText() {
            @Override
            public void afterTextChanged(Editable s) {
                super.afterTextChanged(s);
                viewModel.changeUsername(s.toString());
                if (binding.password.getText().length() >= 5 && binding.username.getText().length() >= 3 && binding.username.getText().toString().matches("[0-9a-zA-Z]+") && (binding.username.getText().toString().charAt(0) + "").matches("[0-9]+")) {
                    binding.login.setEnabled(true);
                } else {
                    binding.login.setEnabled(false);
                }
                if (binding.error.getVisibility() == View.VISIBLE) {
                    binding.error.setVisibility(View.GONE);
                }
            }
        });


        binding.password.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                viewModel.changePassword(charSequence.toString());
                if (binding.password.getText().length() >= 5 && binding.username.getText().length() >= 3 && binding.username.getText().toString().matches("[0-9a-zA-Z]+") && (binding.username.getText().toString().charAt(0) + "").matches("[0-9]+")) {
                    binding.login.setEnabled(true);
                } else {
                    binding.login.setEnabled(false);
                }
                if (binding.error.getVisibility() == View.VISIBLE) {
                    binding.error.setVisibility(View.GONE);
                }
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        binding.login.setOnClickListener(view1 -> {
            viewModel.confirm();

        });

        subscribe(viewModel);
    }



    private void subscribe(LoginViewModel viewModel) {
        viewModel.errorLiveData.observe(getViewLifecycleOwner(), error -> {
            Toast.makeText(getContext(), error, Toast.LENGTH_SHORT).show();
        });

        viewModel.openListLiveData.observe(getViewLifecycleOwner(), username -> {
            final View view = getView();
            Toast.makeText(getContext(), "FFFFFFFFF", Toast.LENGTH_SHORT).show();
            if (view == null) return;
            preferenceManager.putString(Constants.KEY_USER_USERNAME, username);
            preferenceManager.putString(Constants.KEY_USER_ROLE, username);
            view.getRootView().findViewById(R.id.nav_view).setVisibility(View.VISIBLE);
            Navigation.findNavController(view).navigate(R.id.action_loginFragment_to_informationFragment);
        });
    }

    @Override
    public void onDestroyView() {
        binding = null;
        super.onDestroyView();
    }
}
