package ru.myitschool.work.ui.profile;

import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.Navigation;

import ru.myitschool.work.R;
import ru.myitschool.work.databinding.FragmentProfileBinding;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.UserEntity;
import ru.myitschool.work.ui.History;
import ru.myitschool.work.utils.Constants;
import ru.myitschool.work.utils.PreferenceManager;
import ru.myitschool.work.utils.Utils;

public class ProfileFragment extends Fragment {
    private static final String KEY_USER_USERNAME = "username";

    private FragmentProfileBinding binding;
    private PreferenceManager preferenceManager;
    private ProfileViewModel viewModel;

    public ProfileFragment() {
        super(R.layout.fragment_profile);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        binding = FragmentProfileBinding.bind(view);
        preferenceManager = new PreferenceManager(this.getContext());
        viewModel = new ViewModelProvider(this).get(ProfileViewModel.class);
        viewModel.stateLiveData.observe(getViewLifecycleOwner(), state -> {
            final UserEntity userEntity = state.getUser();

                binding.username.setText(userEntity.getUsername());
                binding.position.setText(userEntity.getPosition());
                binding.lastTime.setText(userEntity.getName());

        });

        binding.exit.setOnClickListener(view2 -> {
            preferenceManager.clear();
            view.getRootView().findViewById(R.id.nav_view).setVisibility(View.GONE);
            Navigation.findNavController(view).navigate(R.id.action_profileFragment_to_informationFragment);
        });

        viewModel.load(preferenceManager.getString(Constants.KEY_USER_USERNAME));

    }

    @Override
    public void onDestroyView() {
        binding = null;
        super.onDestroyView();
    }

    public static Bundle getBundle(@NonNull String id) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_USER_USERNAME, id);
        return bundle;
    }
}


